﻿using System;
using System.Collections.Generic;
using System.Linq;

using Hjg.Pngcs;

using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.MathTools;
using Vintagestory.Common;

namespace Automap
{
	public class FlatRenderer : AChunkRenderer
	{
		public const string Name = @"Flat";

		/// <summary>
		/// Renders shards similar to the OLD VS (1.8) version, plus P.O.I. markings.
		/// </summary>
		/// <param name="clientAPI">Client API.</param>
		/// <param name="logger">Logger.</param>
		public FlatRenderer(ICoreClientAPI clientAPI, ILogger logger, bool seasonalColor) : base(clientAPI, logger, seasonalColor)
		{

		}

		public override void GenerateChunkPngShard(Vec2i chunkPos, IMapChunk mc, ColumnMeta targetColMeta, ref ColumnsMetadata allCols, out uint pixelCount)
		{
		pixelCount = 0;
		BlockPos tmpPos = new BlockPos( );
		Vec2i localpos = new Vec2i( );

		var chunksColumn = new IWorldChunk[ClientAPI.World.BlockAccessor.MapSizeY / chunkSize];

		//pre-create PNG line slices...
		ImageLine[ ] lines = Enumerable.Repeat(new object( ), chunkSize).Select(l => new ImageLine(this.PngWriter.ImgInfo)).ToArray( );

		int topChunkY = targetColMeta.YMax / chunkSize;

		for (int chunkY = 0; chunkY <= topChunkY; chunkY++) {
		chunksColumn[chunkY] = ClientAPI.World.BlockAccessor.GetChunk(chunkPos.X, chunkY, chunkPos.Y);
		WorldChunk ownChunk = chunksColumn[chunkY] as WorldChunk;
		if (ownChunk != null) {
		if (ownChunk.IsPacked( )) ownChunk.Unpack( );//Gah - probably done already by chunk processor
		}
		else {
		Logger.Warning("CHUNK A.W.O.L. : X{0} Y{1} Z{2}	- Missing slice FOR COLUMN", chunkPos.X, chunkY, chunkPos.Y);
		//return;
		}
		}

		for (int pixelIndex = 0; pixelIndex < (chunkSize * chunkSize); pixelIndex++) {
		MapUtil.PosInt2d(pixelIndex, chunkSize, localpos);
		int localX = localpos.X;
		int localZ = localpos.Y;
		ushort localY = targetColMeta.HeightMap[localX, localZ];

		int localChunkY = localY / chunkSize;
		if (localChunkY >= (chunksColumn.Length)) continue;//Out of range!
		if (chunksColumn[localChunkY] == null) continue;//BIG Gaps!

		int blockId = chunksColumn[localChunkY].MaybeBlocks[MapUtil.Index3d(localX, (localY % chunkSize), localZ, chunkSize, chunkSize)];

		Block block = ClientAPI.World.Blocks[blockId];

		tmpPos.Set(chunkSize * chunkPos.X + localpos.X, localY, chunkSize * chunkPos.Y + localpos.Y);

		int red = 0, green = 0, blue = 0;

		ExtractBlockColor(tmpPos, block, 1.0f, out red, out green, out blue);

		//============ POI Population =================
		if (BlockID_Designators.ContainsKey(blockId)) {
		var desig = BlockID_Designators[blockId];

		if (desig.Enabled) {
		red = desig.OverwriteColor.R;
		green = desig.OverwriteColor.G;
		blue = desig.OverwriteColor.B;
		}
		}

		ImageLineHelper.SetPixel(lines[localZ], localX, red, green, blue);
		pixelCount++;
		}

		for (int row = 0; row < this.PngWriter.ImgInfo.Rows; row++) {
		this.PngWriter.WriteRow(lines[row], row);
		}

		this.PngWriter.End( );
		}
	}
}